@@ -199,6 +199,7 @@ mi_decl_cache_align _Atomic(mi_submap_t)* _mi_page_map;
 static size_t       mi_page_map_count;
 static void*        mi_page_map_max_address;
 static mi_memid_t   mi_page_map_memid;
+static mi_lock_t    mi_page_map_lock;
 
 // divide the main map in 64 (`MI_BFIELD_BITS`) parts commit those parts on demand
 static _Atomic(mi_bfield_t)  mi_page_map_commit;
@@ -279,7 +280,8 @@ bool _mi_page_map_init(void) {
     return false;
   }
   mi_atomic_store_ptr_release(mi_page_t*, &_mi_page_map[0], sub0);
-
+  mi_lock_init(&mi_page_map_lock);             // initialize late in case the lock init causes allocation
+  
   mi_assert_internal(_mi_ptr_page(NULL)==NULL);
   return true;
 }
@@ -289,6 +291,7 @@ void _mi_page_map_unsafe_destroy(mi_subproc_t* subproc) {
   mi_assert_internal(subproc != NULL);
   mi_assert_internal(_mi_page_map != NULL);
   if (_mi_page_map == NULL) return;
+  mi_lock_done(&mi_page_map_lock);  
   for (size_t idx = 1; idx < mi_page_map_count; idx++) {  // skip entry 0 (as we allocate that submap at the end of the page_map)
     // free all sub-maps
     if (mi_page_map_is_committed(idx, NULL)) {
@@ -317,19 +320,28 @@ mi_decl_nodiscard static bool mi_page_map_ensure_submap_at(size_t idx, mi_submap
   }
   if mi_unlikely(sub == NULL) {
     // sub map not yet allocated, alloc now
-    mi_memid_t memid;
-    const size_t submap_size = MI_PAGE_MAP_SUB_SIZE;
-    sub = (mi_submap_t)_mi_os_zalloc(submap_size, &memid);
-    if (sub==NULL) {
-      _mi_warning_message("internal error: unable to extend the page map\n");
-      return false;
-    }
-    mi_submap_t expect = NULL;
-    if (!mi_atomic_cas_ptr_strong_acq_rel(mi_page_t*, &_mi_page_map[idx], &expect, sub)) {
-      // another thread already allocated it.. free and continue
-      _mi_os_free(sub, submap_size, memid);
-      sub = expect;
+    mi_lock(&mi_page_map_lock) 
+    {
+      sub = mi_atomic_load_ptr_acquire(mi_page_t*, &_mi_page_map[idx]); // reload
+      if (sub==NULL) // not yet allocated by another thread?      
+      {
+        mi_memid_t memid;
+        const size_t submap_size = MI_PAGE_MAP_SUB_SIZE;
+        sub = (mi_submap_t)_mi_os_zalloc(submap_size, &memid);        
+        if (sub==NULL) {
+          _mi_warning_message("internal error: unable to extend the page map\n");          
+        }
+        else {
+          mi_submap_t expect = NULL;
+          if (!mi_atomic_cas_ptr_strong_acq_rel(mi_page_t*, &_mi_page_map[idx], &expect, sub)) {
+            // another thread already allocated it.. free and continue
+            _mi_os_free(sub, submap_size, memid);
+            sub = expect;
+          }
+        }
+      }
     }
+    if (sub==NULL) return false; // unable to allocate the submap..
   }
   mi_assert_internal(sub!=NULL);
   *submap = sub;
